#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>

#define DELAY_SECVENTA		90
#define NUM_SECVENTE		7

#define NUM_NOTE		8
#define DURATA_NOTA		10
 
float frecventa_nota[8] = {
	261.63, 293.66, 329.63, 349.23, 392.0, 440.0, 493.88, 523.25
};


/* Genereaza o nota folosind buzzer-ul. */
void buzzer_reda(int nota) {
	float perioada_nota = 100000 / frecventa_nota[nota];
	int i, j;

	for (i = 0; i < DURATA_NOTA; ++i) {
		PORTD |= (1 << PD5);

		/*
		 * Nu putem folosi variable ca argument pentru functia
		 * _delay_us(), asa ca folosim acest "workaround".
		 */
		for (j = 0; j < perioada_nota / 2; ++j)
			_delay_us(1);

		PORTD &= ~(1 << PD5);

		/*
		 * Nu putem folosi variable ca argument pentru functia
		 * _delay_us(), asa ca folosim acest "workaround".
		 */
		for (j = 0 ; j < perioada_nota / 2; ++j)
			_delay_us(1);
	}
}

/*
 * Aprinde LED-urile legate la porturi in functie de secventele alese.
 * Secventa va fi selectata la apasarea butonului PD6.
 */
void reda_secventa(void) {
	/* Numar secventa curenta. */
	int secventa = 0;
	int i = 0;
	int aux = 0;
	
	int nota_curenta = -1;

	/* Setez pinul PD5 ca iesire. */
	DDRD |= (1 << PD5);

	/* Opresc buzzer-ul. */
	PORTD &= ~(1 << PD5);
	
	/* Setez pinii PD6 ca pini de intrare. */
	DDRD &= ~(1 << PD6);


	/* Setez toti pinii porturilor ca pini de iesire. */
	DDRA = 0xFF;
	DDRC = 0xFF;
	DDRD |= (1 << PD0)|(1 << PD1)|(1 << PD4)|(1 << PD7);
	DDRB = 0xFF;

	/* Sting toate LED-urile*/
	PORTA = 0x00;
	PORTC = 0x00;
	PORTD &= ~((1 << PD0)|(1 << PD1)|(1 << PD4)|(1 << PD7));
	PORTB = 0x00;
	int nu_apasat = 0;
	int j;
	
	while(1) {
		/* Verific daca PD6 este apasat. */
		if ((PIND & (1 << PD6)) == 0) {
			if (nu_apasat == 0)
			{
				nu_apasat = 1;
			
				secventa = (secventa + 1)%NUM_SECVENTE;
				if (nota_curenta < NUM_NOTE - 1)
					++nota_curenta;
				/* Sting LED-urile pentru a pregati noua secventa. */
				PORTA = 0x00;
				PORTC = 0x00;
				PORTD &= ~((1 << PD0)|(1 << PD1)|(1 << PD4)|(1 << PD7));
				PORTB = 0x00;
				i = 0;
			}
		} else
		{
			if ((PIND & (1 << PD6)) != 0) {
			nu_apasat = 0;
			}
		}

		/* Iterez prin secvente in functie de cea selectata. */
		switch (secventa) {
			
			case 0:
				i %= 4;
				if(i == 0) {
					PORTA =(1<< 0)|(1<< 4);
					PORTC =(1<< 0)|(1<< 4);
					PORTD =(1<< 0);
					PORTB =(1<< 0)|(1<< 4);
				} else if(i == 1) {
					PORTA =(1<< 1)|(1<< 5);
					PORTC =(1<< 1)|(1<< 5);
					PORTD =(1<< 1);
					PORTB =(1<< 1)|(1<< 5);
					
				} else if(i == 2) {
					PORTA =(1<< 2)|(1<< 6);
					PORTC =(1<< 2)|(1<< 6);
					PORTD =(1<< 4);
					PORTB =(1<< 2)|(1<< 6);
				} else if(i == 3) {
					PORTA =(1<< 3)|(1<< 7);
					PORTC =(1<< 3)|(1<< 7);
					PORTD =(1<< 7);
					PORTB =(1<< 3)|(1<< 7);
				}
					
				break;
			
			case 1:
					if(i == 24 ) {
						PORTA = 0x00;
						PORTC = 0x00;
						PORTD &= ~((1 << PD0)|(1 << PD1)|(1 << PD4)|(1 << PD7));
						PORTB = 0x00;
						i = 0;
					}
					if(i < 3) 
						PORTB =(1<<7)| (1<<6)|(1<<5)|(1<<4)|(1<<3)|(1<<2)|(1<<1)|(1<<0);
					if (i >= 3 && i < 6) {
						PORTD = (1<<7)|(1<<4)|(1<<1)|(1<<0);
						PORTC = (1<<7)|(1<<6)|(1<<5)|(1<<4);
					}
					if (i >= 6 && i < 9 ) {
						PORTC = (1<<7)|(1<<6)|(1<<5)|(1<<4)|(1<<3)|(1<<2)|(1<<1)|(1<<0);
						PORTA = (1<<7)| (1<<6)|(1<<5)|(1<<4);
					}
					if (i >= 9 && i < 12) {
						PORTA = (1<<7)| (1<<6)|(1<<5)|(1<<4)|(1<<3)|(1<<2)|(1<<1)|(1<<0);
					}
					
					if( i >= 12 && i < 15) 
						PORTA &= ~((1 << PA0)|(1 << PA1)|(1 << PA2)|(1 << PA3));;
					if( i >= 15 && i < 18) {
						PORTA &= ~((1 << PA4)|(1 << PA5)|(1 << PA6)|(1 << PA7));
						PORTC &= ~((1 << PC0)|(1 << PC1)|(1 << PC2)|(1 << PC3));
					}
					if( i >= 18 && i < 21) {
						PORTC &= ~((1 << PC4)|(1 << PC5)|(1 << PC6)|(1 << PC7));
						PORTD &= ~((1 << PD0)|(1 << PD1)|(1 << PD4)|(1 << PD7));
					}
					if ( i >= 21 && i < 24)
						PORTB =0x00;
				break;
			
			case 2:
				if(i == 16) {
					PORTA = 0x00;
					PORTC = 0x00;
					PORTD &= ~((1 << PD0)|(1 << PD1)|(1 << PD4)|(1 << PD7));
					PORTB = 0x00;
					i = 0;
				}
				if( i < 8) {
					PORTA |=(1<< i);
					PORTB |=(1<< (7-i));
				} else if( i>= 8 && i< 12) {
					aux = i - 8;
					PORTC |= (1 << aux);
					if ( i == 8)
						PORTD |= (1<<7);
					if( i == 9)
						PORTD |= (1<<4);
					if( i == 10)
						PORTD |= (1<<1);
					if( i == 11)
						PORTD |= (1<<0);
				} else if(i >= 12 && i < 16) {
					PORTC |=(1<<(i-8));
				}
				break;

			case 3:
				i %= 7;
				if(i == 0) {
					PORTA =(1<< 5);
					PORTC =(1<< 3);
					PORTD =(1<< 4);
					PORTB =(1<< 5)|(1<< 6);
				} else if(i == 1) {
					PORTA =(1<< 4);
					PORTC =(1<< 2);
					PORTD =(1<< 1);
					PORTB =(1<< 4);
					
				} else if(i == 2) {
					PORTA =(1<< 3);
					PORTC =(1<< 1);
					PORTD =(1<< 0);
					PORTB =(1<< 3);
				} else if(i == 3) {
					PORTA =(1<< 2);
					PORTC =(1<< 0)|(1<< 7);
					PORTB =(1<< 2);
				} else if(i == 4) {
					PORTA =(1<< 1)|(1<<7);
					PORTC =(1<< 5);
					PORTB =(1<< 0);
				} else if(i == 5) {
					PORTA = (1<<0)|(1<<6);
					PORTC = (1<<4);
					PORTD = (1<<7);
					PORTB = (1<<7);
				} else if(i == 6) {
					PORTC =(1<< 6);
					PORTB =(1<< 1);
				}
				break;
			case 4:
				if(i < 8) {
					PORTA =(1<< 0)|(1<< 4);
					PORTC =(1<< 0)|(1<< 4);
					PORTD =(1<< 0);
					PORTB =(1<< 0)|(1<< 4);
					PORTA |=(1<< 2)|(1<< 6);
					PORTC |=(1<< 2)|(1<< 6);
					PORTD |=(1<< 4);
					PORTB |=(1<< 2)|(1<< 6);
				} else if(i >=8 && i < 16) {
					PORTA =(1<< 1)|(1<< 5);
					PORTC =(1<< 1)|(1<< 5);
					PORTD =(1<< 1);
					PORTB =(1<< 1)|(1<< 5);
					PORTA |=(1<< 3)|(1<< 7);
					PORTC |=(1<< 3)|(1<< 7);
					PORTD |=(1<< 7);
					PORTB |=(1<< 3)|(1<< 7);
				} else if ( i == 16) {
						PORTA = 0x00;
						PORTC = 0x00;
						PORTD &= ~((1 << PD0)|(1 << PD1)|(1 << PD4)|(1 << PD7));
						PORTB = 0x00;
						i = 0;
				}
				break;
			case 5:
				if( i < 8) {
					PORTA |=(1<< i);
				} else if( i>= 8 && i< 16) {
					aux = i - 8;
					PORTC |= (1 << aux);
				} else if(i >= 16 && i < 20) {
					if ( i == 16)
						PORTD |= (1<<0);
					if( i == 17)
						PORTD |= (1<<1);
					if( i == 18)
						PORTD |= (1<<4);
					if( i == 19)
						PORTD |= (1<<7);
				} else if (i >= 20 && i < 28) {
					PORTB |=(1<<(i-20));
				}
				if( i >= 28 && i < 36) {
					if(i == 28) {
						PORTA = 0x00;
						PORTC = 0x00;
						PORTD &= ~((1 << PD0)|(1 << PD1)|(1 << PD4)|(1 << PD7));
						PORTB = 0x00;
					}
					PORTB |=(1<<(35-i));
				}
				if ( i >= 36 && i < 40) {
					if ( i == 36)
						PORTD |= (1<<7);
					if( i == 37)
						PORTD |= (1<<4);
					if( i == 38)
						PORTD |= (1<<1);
					if( i == 39)
						PORTD |= (1<<0);
				}
				if ( i >= 40 && i < 48)
					PORTC |= (1 << (47-i));
				if (i >= 48 && i < 56)
					PORTA |= (1 << (55-i));
				if ( i == 56) {
						PORTA = 0x00;
						PORTC = 0x00;
						PORTD &= ~((1 << PD0)|(1 << PD1)|(1 << PD4)|(1 << PD7));
						PORTB = 0x00;
						i = 0;
						PORTA =(1<< 0);
				}
				break;
			case 6:
				for (j = 0; j < 8; j++) {
					PORTA |=(1 << j);
					PORTD = (1 << 0)|(1 << 1)|(1 << 4)|(1 << 7);
					PORTC |=(1 << j);
					PORTB |=(1 << j);
				}
				if (nota_curenta != -1) {
					/* Redam nota. */
					buzzer_reda(nota_curenta);
				}
				break;
			default:
				break;
		}

		i++; 
		_delay_ms(DELAY_SECVENTA);
	}
}

int main(void) {
	
	reda_secventa();
	return 0;
}
